
# Slim 4 Boilerplate

Slim 4 starter kit API. Describes endpoints for plugins `session` and for models `Bookmark`, `Place`, `Place Category`, `Service`, `User`.

## Bookmark

### Create `POST /bookmark`

**Request** *(application/json)*

```json
{
    "place": 370
}
```

**Response 201** *(application/json)*

```json
{
    "_id": 131,
    "created_at": 1581014802275,
    "updated_at": 1581014802275,
    "owner": 878,
    "place": 842
}
```

**Response 409** *(application/json)*

Another bookmark with this unique key exists.

```json
{
    "statusCode": 409,
    "error": "Conflict",
    "message": "Duplicate key"
}
```

### Read `GET /bookmark/{bookmark_id}`

**Parameters**

+ `bookmark_id` *(number)* - The id of the bookmark.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "_id": 468,
    "created_at": 1581014802276,
    "updated_at": 1581014802276,
    "owner": {
        "_id": 268,
        "created_at": 1581014802276,
        "updated_at": 1581014802276,
        "email": "email@example.com",
        "name": "Name",
        "role": "Role",
        "banned": true,
        "last_connected_at": 1581014802276
    },
    "place": {
        "_id": 32,
        "created_at": 1581014802276,
        "updated_at": 1581014802276,
        "name": "Name",
        "description": "Description",
        "categories": [
            471
        ],
        "address_1": "Address 1",
        "address_2": "Address 2",
        "latitude": -41.458,
        "longitude": -109.374,
        "phone": "Phone",
        "website_url": "Website Url",
        "services": [
            794
        ],
        "timetable": "Timetable",
        "owner": 665,
        "disabled": true
    }
}
```

**Response 404** *(application/json)*

No bookmark with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "Bookmark not found"
}
```

### Update `PATCH /bookmark/{bookmark_id}`

**Parameters**

+ `bookmark_id` *(number)* - The id of the bookmark.

**Request** *(application/json)*

```json
{
    "place": 569
}
```

**Response 204** *(application/json)*

**Response 404** *(application/json)*

No bookmark with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "Bookmark not found"
}
```

**Response 409** *(application/json)*

Another bookmark with this unique key exists.

```json
{
    "statusCode": 409,
    "error": "Conflict",
    "message": "Duplicate key"
}
```

### Delete `DELETE /bookmark/{bookmark_id}`

**Parameters**

+ `bookmark_id` *(number)* - The id of the bookmark.

**Request** *(application/json)*

**Response 204** *(application/json)*

**Response 404** *(application/json)*

No bookmark with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "Bookmark not found"
}
```

### List `GET /bookmark{?owner,place,_page,_limit,_sort,_order}`

**Parameters**

+ `owner` *(optional, string)* - Id of owner.
+ `place` *(optional, string)* - Id of place.
+ `_page` *(number)* - Page number. Greater or equal than 0.
+ `_limit` *(number)* - Required results length. Greater or equal than 1. Lower or equal than 100.
+ `_sort` *(optional, string)* - Required sort. Can be `created_at`, `updated_at`.
+ `_order` *(optional, string)* - Required sort order. Can be `asc` or `desc`.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "page": 0,
    "limit": 10,
    "count": 10,
    "total": 32,
    "items": [
        {
            "_id": 866,
            "created_at": 1581014802278,
            "updated_at": 1581014802278,
            "owner": 900,
            "place": 54
        }
    ]
}
```

### Count `GET /bookmark/count{?owner,place}`

**Parameters**

+ `owner` *(optional, string)* - Id of owner.
+ `place` *(optional, string)* - Id of place.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "total": 32
}
```



## Place

### Create `POST /place`

**Request** *(application/json)*

```json
{
    "name": "Name",
    "description": "Description",
    "categories": [
        37
    ],
    "address_1": "Address 1",
    "address_2": "Address 2",
    "phone": "Phone",
    "website_url": "Website Url",
    "services": [
        840
    ],
    "timetable": "Timetable",
    "disabled": true
}
```

**Response 201** *(application/json)*

```json
{
    "_id": 436,
    "created_at": 1581014802279,
    "updated_at": 1581014802279,
    "name": "Name",
    "description": "Description",
    "categories": [
        818
    ],
    "address_1": "Address 1",
    "address_2": "Address 2",
    "latitude": 24.892,
    "longitude": 58.412,
    "phone": "Phone",
    "website_url": "Website Url",
    "services": [
        480
    ],
    "timetable": "Timetable",
    "owner": 751,
    "disabled": true
}
```

### Read `GET /place/{place_id}`

**Parameters**

+ `place_id` *(number)* - The id of the place.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "_id": 267,
    "created_at": 1581014802279,
    "updated_at": 1581014802279,
    "name": "Name",
    "description": "Description",
    "categories": [
        {
            "_id": 463,
            "created_at": 1581014802279,
            "updated_at": 1581014802279,
            "name": "Name",
            "description": "Description"
        }
    ],
    "address_1": "Address 1",
    "address_2": "Address 2",
    "latitude": -4.732,
    "longitude": -79.222,
    "phone": "Phone",
    "website_url": "Website Url",
    "services": [
        {
            "_id": 125,
            "created_at": 1581014802279,
            "updated_at": 1581014802279,
            "name": "Name",
            "description": "Description"
        }
    ],
    "timetable": "Timetable",
    "owner": {
        "_id": 965,
        "created_at": 1581014802280,
        "updated_at": 1581014802280,
        "email": "email@example.com",
        "name": "Name",
        "role": "Role",
        "banned": true,
        "last_connected_at": 1581014802280
    },
    "disabled": true
}
```

**Response 404** *(application/json)*

No place with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "Place not found"
}
```

### Update `PATCH /place/{place_id}`

**Parameters**

+ `place_id` *(number)* - The id of the place.

**Request** *(application/json)*

```json
{
    "name": "New Name",
    "description": "New Description",
    "categories": [
        686
    ],
    "address_1": "New Address 1",
    "address_2": "New Address 2",
    "phone": "New Phone",
    "website_url": "New Website Url",
    "services": [
        797
    ],
    "timetable": "New Timetable",
    "disabled": false
}
```

**Response 204** *(application/json)*

**Response 404** *(application/json)*

No place with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "Place not found"
}
```

### Delete `DELETE /place/{place_id}`

This action will also remove references from `Bookmark.place`.

**Parameters**

+ `place_id` *(number)* - The id of the place.

**Request** *(application/json)*

**Response 204** *(application/json)*

**Response 404** *(application/json)*

No place with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "Place not found"
}
```

### List `GET /place{?name,categories,latitude,latitude__min,latitude__max,longitude,longitude__min,longitude__max,services,owner,disabled,_page,_limit,_sort,_order}`

**Parameters**

+ `name` *(optional, string)* - Value for name.
+ `categories` *(optional, string)* - Id of categories. Can be multiple.
+ `latitude` *(optional, number)* - Value for latitude.
+ `latitude__min` *(optional, number)* - Minimum value for latitude.
+ `latitude__max` *(optional, number)* - Maximum value for latitude.
+ `longitude` *(optional, number)* - Value for longitude.
+ `longitude__min` *(optional, number)* - Minimum value for longitude.
+ `longitude__max` *(optional, number)* - Maximum value for longitude.
+ `services` *(optional, string)* - Id of services. Can be multiple.
+ `owner` *(optional, string)* - Id of owner.
+ `disabled` *(optional, boolean)* - Value for disabled.
+ `_page` *(number)* - Page number. Greater or equal than 0.
+ `_limit` *(number)* - Required results length. Greater or equal than 1. Lower or equal than 100.
+ `_sort` *(optional, string)* - Required sort. Can be `created_at`, `updated_at`, `name`, `latitude`, `longitude`.
+ `_order` *(optional, string)* - Required sort order. Can be `asc` or `desc`.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "page": 0,
    "limit": 10,
    "count": 10,
    "total": 32,
    "items": [
        {
            "_id": 636,
            "created_at": 1581014802280,
            "updated_at": 1581014802280,
            "name": "Name",
            "description": "Description",
            "categories": [
                {
                    "_id": 454,
                    "created_at": 1581014802281,
                    "updated_at": 1581014802281,
                    "name": "Name",
                    "description": "Description"
                }
            ],
            "address_1": "Address 1",
            "address_2": "Address 2",
            "latitude": -49.106,
            "longitude": 17.508,
            "phone": "Phone",
            "website_url": "Website Url",
            "services": [
                558
            ],
            "timetable": "Timetable",
            "owner": 170,
            "disabled": true
        }
    ]
}
```

### Count `GET /place/count{?name,categories,latitude,latitude__min,latitude__max,longitude,longitude__min,longitude__max,services,owner,disabled}`

**Parameters**

+ `name` *(optional, string)* - Value for name.
+ `categories` *(optional, string)* - Id of categories. Can be multiple.
+ `latitude` *(optional, number)* - Value for latitude.
+ `latitude__min` *(optional, number)* - Minimum value for latitude.
+ `latitude__max` *(optional, number)* - Maximum value for latitude.
+ `longitude` *(optional, number)* - Value for longitude.
+ `longitude__min` *(optional, number)* - Minimum value for longitude.
+ `longitude__max` *(optional, number)* - Maximum value for longitude.
+ `services` *(optional, string)* - Id of services. Can be multiple.
+ `owner` *(optional, string)* - Id of owner.
+ `disabled` *(optional, boolean)* - Value for disabled.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "total": 32
}
```



## Place Category

### Create `POST /place-category`

**Request** *(application/json)*

```json
{
    "name": "Name",
    "description": "Description"
}
```

**Response 201** *(application/json)*

```json
{
    "_id": 323,
    "created_at": 1581014802281,
    "updated_at": 1581014802281,
    "name": "Name",
    "description": "Description"
}
```

### Read `GET /place-category/{place_category_id}`

**Parameters**

+ `place_category_id` *(number)* - The id of the place category.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "_id": 135,
    "created_at": 1581014802281,
    "updated_at": 1581014802281,
    "name": "Name",
    "description": "Description"
}
```

**Response 404** *(application/json)*

No place category with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "Place Category not found"
}
```

### Update `PATCH /place-category/{place_category_id}`

**Parameters**

+ `place_category_id` *(number)* - The id of the place category.

**Request** *(application/json)*

```json
{
    "name": "New Name",
    "description": "New Description"
}
```

**Response 204** *(application/json)*

**Response 404** *(application/json)*

No place category with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "Place Category not found"
}
```

### Delete `DELETE /place-category/{place_category_id}`

This action will also remove references from `Place.categories`.

**Parameters**

+ `place_category_id` *(number)* - The id of the place category.

**Request** *(application/json)*

**Response 204** *(application/json)*

**Response 404** *(application/json)*

No place category with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "Place Category not found"
}
```

### List `GET /place-category{?name,_page,_limit,_sort,_order}`

**Parameters**

+ `name` *(optional, string)* - Value for name.
+ `_page` *(number)* - Page number. Greater or equal than 0.
+ `_limit` *(number)* - Required results length. Greater or equal than 1. Lower or equal than 100.
+ `_sort` *(optional, string)* - Required sort. Can be `created_at`, `updated_at`, `name`.
+ `_order` *(optional, string)* - Required sort order. Can be `asc` or `desc`.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "page": 0,
    "limit": 10,
    "count": 10,
    "total": 32,
    "items": [
        {
            "_id": 427,
            "created_at": 1581014802281,
            "updated_at": 1581014802281,
            "name": "Name",
            "description": "Description"
        }
    ]
}
```

### Count `GET /place-category/count{?name}`

**Parameters**

+ `name` *(optional, string)* - Value for name.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "total": 32
}
```



## Service

### Create `POST /service`

**Request** *(application/json)*

```json
{
    "name": "Name",
    "description": "Description"
}
```

**Response 201** *(application/json)*

```json
{
    "_id": 437,
    "created_at": 1581014802282,
    "updated_at": 1581014802282,
    "name": "Name",
    "description": "Description"
}
```

**Response 409** *(application/json)*

Another service with this unique key exists.

```json
{
    "statusCode": 409,
    "error": "Conflict",
    "message": "Duplicate key"
}
```

### Read `GET /service/{service_id}`

**Parameters**

+ `service_id` *(number)* - The id of the service.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "_id": 609,
    "created_at": 1581014802282,
    "updated_at": 1581014802282,
    "name": "Name",
    "description": "Description"
}
```

**Response 404** *(application/json)*

No service with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "Service not found"
}
```

### Update `PATCH /service/{service_id}`

**Parameters**

+ `service_id` *(number)* - The id of the service.

**Request** *(application/json)*

```json
{
    "name": "New Name",
    "description": "New Description"
}
```

**Response 204** *(application/json)*

**Response 404** *(application/json)*

No service with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "Service not found"
}
```

**Response 409** *(application/json)*

Another service with this unique key exists.

```json
{
    "statusCode": 409,
    "error": "Conflict",
    "message": "Duplicate key"
}
```

### Delete `DELETE /service/{service_id}`

This action will also remove references from `Place.services`.

**Parameters**

+ `service_id` *(number)* - The id of the service.

**Request** *(application/json)*

**Response 204** *(application/json)*

**Response 404** *(application/json)*

No service with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "Service not found"
}
```

### List `GET /service{?name,_page,_limit,_sort,_order}`

**Parameters**

+ `name` *(optional, string)* - Value for name.
+ `_page` *(number)* - Page number. Greater or equal than 0.
+ `_limit` *(number)* - Required results length. Greater or equal than 1. Lower or equal than 100.
+ `_sort` *(optional, string)* - Required sort. Can be `created_at`, `updated_at`, `name`.
+ `_order` *(optional, string)* - Required sort order. Can be `asc` or `desc`.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "page": 0,
    "limit": 10,
    "count": 10,
    "total": 32,
    "items": [
        {
            "_id": 70,
            "created_at": 1581014802282,
            "updated_at": 1581014802282,
            "name": "Name",
            "description": "Description"
        }
    ]
}
```

### Count `GET /service/count{?name}`

**Parameters**

+ `name` *(optional, string)* - Value for name.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "total": 32
}
```



## User

### Create `POST /user`

**Request** *(application/json)*

```json
{
    "email": "email@example.com",
    "name": "Name",
    "password": "lQSMZKoImTRh",
    "role": "Role",
    "banned": true
}
```

**Response 201** *(application/json)*

```json
{
    "_id": 833,
    "created_at": 1581014802283,
    "updated_at": 1581014802283,
    "email": "email@example.com",
    "name": "Name",
    "role": "Role",
    "banned": true,
    "last_connected_at": 1581014802283
}
```

**Response 409** *(application/json)*

Another user with this unique key exists.

```json
{
    "statusCode": 409,
    "error": "Conflict",
    "message": "Duplicate key"
}
```

### Read `GET /user/{user_id}`

**Parameters**

+ `user_id` *(number)* - The id of the user.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "_id": 664,
    "created_at": 1581014802284,
    "updated_at": 1581014802284,
    "email": "email@example.com",
    "name": "Name",
    "role": "Role",
    "banned": true,
    "last_connected_at": 1581014802284
}
```

**Response 404** *(application/json)*

No user with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "User not found"
}
```

### Update `PATCH /user/{user_id}`

**Parameters**

+ `user_id` *(number)* - The id of the user.

**Request** *(application/json)*

```json
{
    "email": "new.email@example.com",
    "name": "New Name",
    "password": "Rl7uW1tJ3VID",
    "role": "New Role",
    "banned": false
}
```

**Response 204** *(application/json)*

**Response 404** *(application/json)*

No user with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "User not found"
}
```

**Response 409** *(application/json)*

Another user with this unique key exists.

```json
{
    "statusCode": 409,
    "error": "Conflict",
    "message": "Duplicate key"
}
```

### Delete `DELETE /user/{user_id}`

This action will also remove references from `Bookmark.owner`, `Place.owner`.

**Parameters**

+ `user_id` *(number)* - The id of the user.

**Request** *(application/json)*

**Response 204** *(application/json)*

**Response 404** *(application/json)*

No user with this id was found.

```json
{
    "statusCode": 404,
    "error": "Not found",
    "message": "User not found"
}
```

### List `GET /user{?email,name,role,banned,_page,_limit,_sort,_order}`

**Parameters**

+ `email` *(optional, string)* - Value for email.
+ `name` *(optional, string)* - Value for name.
+ `role` *(optional, string)* - Value for role.
+ `banned` *(optional, boolean)* - Value for banned.
+ `_page` *(number)* - Page number. Greater or equal than 0.
+ `_limit` *(number)* - Required results length. Greater or equal than 1. Lower or equal than 100.
+ `_sort` *(optional, string)* - Required sort. Can be `created_at`, `updated_at`, `email`, `name`.
+ `_order` *(optional, string)* - Required sort order. Can be `asc` or `desc`.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "page": 0,
    "limit": 10,
    "count": 10,
    "total": 32,
    "items": [
        {
            "_id": 184,
            "created_at": 1581014802285,
            "updated_at": 1581014802285,
            "email": "email@example.com",
            "name": "Name",
            "role": "Role",
            "banned": true,
            "last_connected_at": 1581014802285
        }
    ]
}
```

### Count `GET /user/count{?email,name,role,banned}`

**Parameters**

+ `email` *(optional, string)* - Value for email.
+ `name` *(optional, string)* - Value for name.
+ `role` *(optional, string)* - Value for role.
+ `banned` *(optional, boolean)* - Value for banned.

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "total": 32
}
```

## Session

### Login `POST /session`

**Request** *(application/json)*

```json
{
    "email": "john@mail.com",
    "password": "pAssW0rd"
}
```

**Response 201** *(application/json)*

Login sucessful. An authentication cookie is also returned.

```json
{
    "_id": 12,
    "name": "John Doe",
    "email": "john@mail.com",
    "role": "user"
}
```

**Response 404** *(application/json)*

Wrong login or wrong password.

```json
{
    "statusCode": 404,
    "error": "Not Found",
    "message": "User not found or wrong password"
}
```

### Logout `DELETE /session`

**Request** *(application/json)*

**Response 204** *(application/json)*

**Response 401** *(application/json)*

Missing or wrong cookie.

```json
{
    "statusCode": 401,
    "error": "Unauthorized",
    "message": "Invalid cookie"
}
```

### Current user `GET /session`

**Request** *(application/json)*

**Response 200** *(application/json)*

```json
{
    "_id": 12,
    "name": "John Doe",
    "email": "john@mail.com",
    "role": "user"
}
```

**Response 401** *(application/json)*

Missing or wrong cookie.

```json
{
    "statusCode": 401,
    "error": "Unauthorized",
    "message": "Missing authentication"
}
```
