<?php

require_once __DIR__ . '/../../../public/initialize.php';
require_once __DIR__ . '/../inc/helper.php';

use \Illuminate\Database\Schema\Blueprint;

/** @var \Illuminate\Database\Schema\Builder $schema */
$schema = $capsule::schema();
$schemaManager = $schema->getConnection()->getDoctrineSchemaManager();

// ==================================
// Create tables & columns
// ==================================

// Create table for model bookmark
if (!$schema->hasTable('bookmark')) {
    $schema->create('bookmark', function (Blueprint $table) {
        $table->increments('_id');
        $table->datetime('created_at');
        $table->datetime('updated_at')->nullable();
        $table->integer('owner', false, true);
        $table->integer('place', false, true);
    });
    Helpers::output('', 'Did create table bookmark');
}

// Create table for model place
if (!$schema->hasTable('place')) {
    $schema->create('place', function (Blueprint $table) {
        $table->increments('_id');
        $table->datetime('created_at');
        $table->datetime('updated_at')->nullable();
        $table->string('name');
        $table->longText('description')->nullable();
        $table->string('address_1');
        $table->string('address_2')->nullable();
        $table->float('latitude');
        $table->float('longitude');
        $table->string('phone')->nullable();
        $table->string('website_url')->nullable();
        $table->text('timetable')->nullable();
        $table->integer('owner', false, true);
        $table->boolean('disabled');
    });
    Helpers::output('', 'Did create table place');
}

// Many-to-many relationships for model place
if (!$schema->hasTable('place__categories')) {
    $schema->create('place__categories', function (Blueprint $table) {
        $table->increments('_id');
        $table->integer('place', false, true);
        $table->integer('place_category', false, true);
    });
    Helpers::output('', 'Did create relationship table place to categories');
}
if (!$schema->hasTable('place__services')) {
    $schema->create('place__services', function (Blueprint $table) {
        $table->increments('_id');
        $table->integer('place', false, true);
        $table->integer('service', false, true);
    });
    Helpers::output('', 'Did create relationship table place to services');
}

// Create table for model place category
if (!$schema->hasTable('place_category')) {
    $schema->create('place_category', function (Blueprint $table) {
        $table->increments('_id');
        $table->datetime('created_at');
        $table->datetime('updated_at')->nullable();
        $table->string('name');
        $table->text('description')->nullable();
    });
    Helpers::output('', 'Did create table place category');
}

// Create table for model service
if (!$schema->hasTable('service')) {
    $schema->create('service', function (Blueprint $table) {
        $table->increments('_id');
        $table->datetime('created_at');
        $table->datetime('updated_at')->nullable();
        $table->string('name');
        $table->text('description')->nullable();
    });
    Helpers::output('', 'Did create table service');
}

// Create table for model user
if (!$schema->hasTable('user')) {
    $schema->create('user', function (Blueprint $table) {
        $table->increments('_id');
        $table->datetime('created_at');
        $table->datetime('updated_at')->nullable();
        $table->string('email');
        $table->string('name');
        $table->string('password');
        $table->string('role');
        $table->boolean('banned');
        $table->datetime('last_connected_at')->nullable();
    });
    Helpers::output('', 'Did create table user');
}
// ==================================
// Create indexes
// ==================================

// Create indexes for model bookmark
$schema->table('bookmark', function (Blueprint $table) {
    global $schemaManager;
    $tableDetails = $schemaManager->listTableDetails($table->getTable());

    if (!$tableDetails->hasIndex('bookmark_created_at')) {
        $table->index('created_at', 'bookmark_created_at');
        Helpers::output('', 'Did create index bookmark_created_at');
    }

    if (!$tableDetails->hasIndex('bookmark_updated_at')) {
        $table->index('updated_at', 'bookmark_updated_at');
        Helpers::output('', 'Did create index bookmark_updated_at');
    }

    if (!$tableDetails->hasIndex('bookmark__uniques')) {
        $table->unique(['owner', 'place', ], 'bookmark__uniques');
        Helpers::output('', 'Did create index bookmark__uniques');
    }

    if (!$tableDetails->hasIndex('bookmark_owner__foreign')) {
        $table->foreign('owner', 'bookmark_owner__foreign')->references('_id')->on('user');
        Helpers::output('', 'Did create index bookmark_owner__foreign');
    }

    if (!$tableDetails->hasIndex('bookmark_place__foreign')) {
        $table->foreign('place', 'bookmark_place__foreign')->references('_id')->on('place');
        Helpers::output('', 'Did create index bookmark_place__foreign');
    }
});

// Create indexes for model place
$schema->table('place', function (Blueprint $table) {
    global $schemaManager;
    $tableDetails = $schemaManager->listTableDetails($table->getTable());

    if (!$tableDetails->hasIndex('place_created_at')) {
        $table->index('created_at', 'place_created_at');
        Helpers::output('', 'Did create index place_created_at');
    }

    if (!$tableDetails->hasIndex('place_updated_at')) {
        $table->index('updated_at', 'place_updated_at');
        Helpers::output('', 'Did create index place_updated_at');
    }

    if (!$tableDetails->hasIndex('place_name')) {
        $table->index('name', 'place_name');
        Helpers::output('', 'Did create index place_name');
    }

    if (!$tableDetails->hasIndex('place_latitude')) {
        $table->index('latitude', 'place_latitude');
        Helpers::output('', 'Did create index place_latitude');
    }

    if (!$tableDetails->hasIndex('place_longitude')) {
        $table->index('longitude', 'place_longitude');
        Helpers::output('', 'Did create index place_longitude');
    }

    if (!$tableDetails->hasIndex('place_disabled')) {
        $table->index('disabled', 'place_disabled');
        Helpers::output('', 'Did create index place_disabled');
    }

    if (!$tableDetails->hasIndex('place_owner__foreign')) {
        $table->foreign('owner', 'place_owner__foreign')->references('_id')->on('user');
        Helpers::output('', 'Did create index place_owner__foreign');
    }
});
// Many-to-many relationships for field place.categories
$schema->table('place__categories', function (Blueprint $table) {
    global $schemaManager;
    $tableDetails = $schemaManager->listTableDetails($table->getTable());

    if (!$tableDetails->hasIndex('place__categories.place')) {
        $table->foreign('place', 'place__categories.place')->references('_id')->on('place')->onDelete('cascade');
        Helpers::output('', 'Did create relationship index for place__categories.place');
    }
    if (!$tableDetails->hasIndex('place__categories.place_category')) {
        $table->foreign('place_category', 'place__categories.place_category')->references('_id')->on('place_category')->onDelete('cascade');
        Helpers::output('', 'Did create relationship index for place__categories.place category');
    }

});
// Many-to-many relationships for field place.services
$schema->table('place__services', function (Blueprint $table) {
    global $schemaManager;
    $tableDetails = $schemaManager->listTableDetails($table->getTable());

    if (!$tableDetails->hasIndex('place__services.place')) {
        $table->foreign('place', 'place__services.place')->references('_id')->on('place')->onDelete('cascade');
        Helpers::output('', 'Did create relationship index for place__services.place');
    }
    if (!$tableDetails->hasIndex('place__services.service')) {
        $table->foreign('service', 'place__services.service')->references('_id')->on('service')->onDelete('cascade');
        Helpers::output('', 'Did create relationship index for place__services.service');
    }

});

// Create indexes for model place category
$schema->table('place_category', function (Blueprint $table) {
    global $schemaManager;
    $tableDetails = $schemaManager->listTableDetails($table->getTable());

    if (!$tableDetails->hasIndex('place_category_created_at')) {
        $table->index('created_at', 'place_category_created_at');
        Helpers::output('', 'Did create index place_category_created_at');
    }

    if (!$tableDetails->hasIndex('place_category_updated_at')) {
        $table->index('updated_at', 'place_category_updated_at');
        Helpers::output('', 'Did create index place_category_updated_at');
    }

    if (!$tableDetails->hasIndex('place_category_name')) {
        $table->index('name', 'place_category_name');
        Helpers::output('', 'Did create index place_category_name');
    }
});

// Create indexes for model service
$schema->table('service', function (Blueprint $table) {
    global $schemaManager;
    $tableDetails = $schemaManager->listTableDetails($table->getTable());

    if (!$tableDetails->hasIndex('service_created_at')) {
        $table->index('created_at', 'service_created_at');
        Helpers::output('', 'Did create index service_created_at');
    }

    if (!$tableDetails->hasIndex('service_updated_at')) {
        $table->index('updated_at', 'service_updated_at');
        Helpers::output('', 'Did create index service_updated_at');
    }

    if (!$tableDetails->hasIndex('service_name')) {
        $table->index('name', 'service_name');
        Helpers::output('', 'Did create index service_name');
    }

    if (!$tableDetails->hasIndex('service__uniques')) {
        $table->unique(['name', ], 'service__uniques');
        Helpers::output('', 'Did create index service__uniques');
    }
});

// Create indexes for model user
$schema->table('user', function (Blueprint $table) {
    global $schemaManager;
    $tableDetails = $schemaManager->listTableDetails($table->getTable());

    if (!$tableDetails->hasIndex('user_created_at')) {
        $table->index('created_at', 'user_created_at');
        Helpers::output('', 'Did create index user_created_at');
    }

    if (!$tableDetails->hasIndex('user_updated_at')) {
        $table->index('updated_at', 'user_updated_at');
        Helpers::output('', 'Did create index user_updated_at');
    }

    if (!$tableDetails->hasIndex('user_email')) {
        $table->index('email', 'user_email');
        Helpers::output('', 'Did create index user_email');
    }

    if (!$tableDetails->hasIndex('user_name')) {
        $table->index('name', 'user_name');
        Helpers::output('', 'Did create index user_name');
    }

    if (!$tableDetails->hasIndex('user_role')) {
        $table->index('role', 'user_role');
        Helpers::output('', 'Did create index user_role');
    }

    if (!$tableDetails->hasIndex('user_banned')) {
        $table->index('banned', 'user_banned');
        Helpers::output('', 'Did create index user_banned');
    }

    if (!$tableDetails->hasIndex('user__uniques')) {
        $table->unique(['email', ], 'user__uniques');
        Helpers::output('', 'Did create index user__uniques');
    }
});
