<?php

namespace App\Models;

use Psr\Http\Message\ServerRequestInterface as Request;

class Bookmark extends BaseModel {

	/**
     * The table associated with the model.
     *
     * @var string
     */
   	protected $table = 'bookmark';

   	/**
     * The attributes that are mass assignable.
     *
     * @var array
     */
   	protected $fillable = array(
        'place',
   	);

   	/**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array(
    );

    /**
     * Get the field created at.
     *
     * @return integer
     */
    public function getCreatedAtAttribute()
    {
        return $this->dateToTimestamp($this->attributes['created_at']);
    }

    /**
     * Set the field created at.
     *
     * @param  string  $value
     */
    public function setCreatedAtAttribute($value)
    {
        $this->attributes['created_at'] = $this->timestampToDate($value);
    }

    /**
     * Get the field updated at.
     *
     * @return integer
     */
    public function getUpdatedAtAttribute()
    {
        return $this->dateToTimestamp($this->attributes['updated_at']);
    }

    /**
     * Set the field updated at.
     *
     * @param  string  $value
     */
    public function setUpdatedAtAttribute($value)
    {
        $this->attributes['updated_at'] = $this->timestampToDate($value);
    }

    /**
     * Get search cursor
     * @param  array $filter
     * @param  Request $request
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public static function getCursor($filter, Request $request) {

        unset($filter['_page']);
        unset($filter['_limit']);
        unset($filter['_order']);
        unset($filter['_sort']);

        $query = Bookmark::query();

        // Filter by related owner
        if (isset($filter['owner'])) {
            $query = $query->where('owner', $filter['owner']);
        }
        else if (!$request->getAttribute('fromAdmin') && $request->getAttribute('userId') !== null) {
            $query = $query->where('owner', '=', $request->getAttribute('userId'));
        }

        // Filter by related place
        if (isset($filter['place'])) {
            $query = $query->where('place', $filter['place']);
        }

        return $query;
    }

    /**
     * Return relationship for owner
     * @return \Illuminate\Database\Eloquent\Relations\HasOne
     */
    public function _owner()
    {
        return $this->hasOne('App\Models\User', '_id', 'owner');
    }

    /**
     * Return relationship for place
     * @return \Illuminate\Database\Eloquent\Relations\HasOne
     */
    public function _place()
    {
        return $this->hasOne('App\Models\Place', '_id', 'place');
    }
}