<?php

namespace App\Models;

use Psr\Http\Message\ServerRequestInterface as Request;

class Service extends BaseModel {

	/**
     * The table associated with the model.
     *
     * @var string
     */
   	protected $table = 'service';

   	/**
     * The attributes that are mass assignable.
     *
     * @var array
     */
   	protected $fillable = array(
        'name',
        'description',
   	);

   	/**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array(
    );

    /**
     * Get the field created at.
     *
     * @return integer
     */
    public function getCreatedAtAttribute()
    {
        return $this->dateToTimestamp($this->attributes['created_at']);
    }

    /**
     * Set the field created at.
     *
     * @param  string  $value
     */
    public function setCreatedAtAttribute($value)
    {
        $this->attributes['created_at'] = $this->timestampToDate($value);
    }

    /**
     * Get the field updated at.
     *
     * @return integer
     */
    public function getUpdatedAtAttribute()
    {
        return $this->dateToTimestamp($this->attributes['updated_at']);
    }

    /**
     * Set the field updated at.
     *
     * @param  string  $value
     */
    public function setUpdatedAtAttribute($value)
    {
        $this->attributes['updated_at'] = $this->timestampToDate($value);
    }

    /**
     * Get search cursor
     * @param  array $filter
     * @param  Request $request
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public static function getCursor($filter, Request $request) {

        unset($filter['_page']);
        unset($filter['_limit']);
        unset($filter['_order']);
        unset($filter['_sort']);

        $query = Service::query();

        // Filter name
        if (isset($filter['name'])) {
            $query = $query->where('name', 'LIKE', "%{$filter['name']}%");
        }

        return $query;
    }
}