<?php

namespace App\Models;

use Psr\Http\Message\ServerRequestInterface as Request;

class User extends BaseModel {

	/**
     * The table associated with the model.
     *
     * @var string
     */
   	protected $table = 'user';

   	/**
     * The attributes that are mass assignable.
     *
     * @var array
     */
   	protected $fillable = array(
        'email',
        'name',
        'password',
        'role',
        'banned',
   	);

   	/**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array(
        'password',
    );

    /**
     * Set the field password.
     *
     * @param  string  $value
     */
    public function setPasswordAttribute($value)
    {
        $this->attributes['password'] = \App\Library\Encryption::hash($value);
    }

    /**
     * Get the field created at.
     *
     * @return integer
     */
    public function getCreatedAtAttribute()
    {
        return $this->dateToTimestamp($this->attributes['created_at']);
    }

    /**
     * Set the field created at.
     *
     * @param  string  $value
     */
    public function setCreatedAtAttribute($value)
    {
        $this->attributes['created_at'] = $this->timestampToDate($value);
    }

    /**
     * Get the field updated at.
     *
     * @return integer
     */
    public function getUpdatedAtAttribute()
    {
        return $this->dateToTimestamp($this->attributes['updated_at']);
    }

    /**
     * Set the field updated at.
     *
     * @param  string  $value
     */
    public function setUpdatedAtAttribute($value)
    {
        $this->attributes['updated_at'] = $this->timestampToDate($value);
    }

    /**
     * Get the field last connected at.
     *
     * @return integer
     */
    public function getLastConnectedAtAttribute()
    {
        return $this->dateToTimestamp($this->attributes['last_connected_at']);
    }

    /**
     * Set the field last connected at.
     *
     * @param  string  $value
     */
    public function setLastConnectedAtAttribute($value)
    {
        $this->attributes['last_connected_at'] = $this->timestampToDate($value);
    }

    /**
     * Get the field banned.
     *
     * @return boolean
     */
    public function getBannedAttribute()
    {
        return $this->intToBoolean($this->attributes['banned']);
    }

    /**
     * Get search cursor
     * @param  array $filter
     * @param  Request $request
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public static function getCursor($filter, Request $request) {

        unset($filter['_page']);
        unset($filter['_limit']);
        unset($filter['_order']);
        unset($filter['_sort']);

        $query = User::query();

        // Filter email
        if (isset($filter['email'])) {
            $query = $query->where('email', $filter['email']);
        }

        // Filter name
        if (isset($filter['name'])) {
            $query = $query->where('name', 'LIKE', "%{$filter['name']}%");
        }

        // Filter role
        if (isset($filter['role'])) {
            $query = $query->where('role', $filter['role']);
        }

        // Filter banned
        if (isset($filter['banned'])) {
            $query = $query->where('banned', static::toBoolean($filter['banned']));
        }

        return $query;
    }
}