<?php 

/**
 * Create bookmark.
 * Returns the created bookmark.
 */

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Valitron\Validator as Validator;
use App\Library\HttpException;

$_func = function (Request $request, Response $response, array $args) {

    //Authentication
    $this->get('session')->verifyCredentials($request);

    $data = $request->getParsedBody();

    //Form validation
    $validator = new Validator($data);
    $validator->rule('required', 'place')->rule('integer', 'place');

    if ($validator->validate()) {

        try {

            $model = new App\Models\Bookmark();
            $model->fill($data);

            // Init internal fields
            $model->setAttribute('created_at', date('Y-m-d H:i:s'));
            $model->setAttribute('updated_at', date('Y-m-d H:i:s'));
            $model->setAttribute('owner', $request->getAttribute('userId'));

            $model->save();

            $response->getBody()->write($model->toJSON());

            return $response->withStatus(201);

        } catch (\Exception $e) {
            if ($e->getCode() == 23000) {
                // Deal with duplicate key error
                throw new HttpException(409, $e->getMessage());
            }
            throw HttpException::internal($e);
        }
    }

    // Bad payload request
    throw HttpException::badRequest($validator);
};

//Define admin route callback
$app->post('/admin/bookmark', $_func);
//Define route callback
$app->post('/bookmark', $_func);
