<?php 

/**
 * Update place category.
 */

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Valitron\Validator as Validator;
use App\Library\HttpException;

$_func = function (Request $request, Response $response, array $args) {

    //Authentication
    $this->get('session')->verifyCredentials($request);

    /** @var App\Models\PlaceCategory $model */
    $model = App\Models\PlaceCategory::find($args['id']);
    if ($model == null) {
        // Entity was not found
        throw new HttpException(404, 'Place Category not found');
    }

    $data = $request->getParsedBody();

    //Form validation
    $validator = new Validator($data);
    $validator->rule('optional', 'name');
    $validator->rule('optional', 'description', true );

    if ($validator->validate()) {

        try {

            $model->fill($data);
            $model->save();

            return $response->withStatus(204);

        } catch (\Exception $e) {

            if ($e->getCode() == 23000) {
                // Deal with duplicate key error
                throw new HttpException(409, $e->getMessage());
            }
            throw HttpException::internal($e);
        }
    }
    // Bad payload request
    throw HttpException::badRequest($validator);
};

//Define admin route callback
$app->patch('/admin/place-category/{id}', $_func);
