<?php 

/**
 * Create place.
 * Returns the created place.
 */

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Valitron\Validator as Validator;
use App\Library\HttpException;

$_func = function (Request $request, Response $response, array $args) {

    //Authentication
    $this->get('session')->verifyCredentials($request);

    $data = $request->getParsedBody();

    //Form validation
    $validator = new Validator($data);
    $validator->rule('required', 'name');
    $validator->rule('required', 'description', true );
    $validator->rule('required', 'categories')->rule('array', 'categories');
    $validator->rule('required', 'address_1');
    $validator->rule('required', 'address_2', true );
    $validator->rule('required', 'phone', true );
    $validator->rule('required', 'website_url', true );
    $validator->rule('required', 'services', true )->rule('array', 'services');
    $validator->rule('required', 'timetable', true );

    // For admins only
    if ($request->getAttribute('fromAdmin')) {
        $validator->rule('required', 'disabled')->rule('boolean', 'disabled');
    }

    if ($validator->validate()) {

        try {

            // Init restricted fields
            if(!$request->getAttribute('fromAdmin')) {
                $data['disabled'] = false;
            }

            $model = new App\Models\Place();
            $model->fill($data);

            // Init internal fields
            $model->setAttribute('created_at', date('Y-m-d H:i:s'));
            $model->setAttribute('updated_at', date('Y-m-d H:i:s'));
            $model->setAttribute('latitude', 0);
            $model->setAttribute('longitude', 0);
            $model->setAttribute('owner', $request->getAttribute('userId'));

            $model->save();
            $model->_categories()->attach($data['categories']);
            $model->_services()->attach($data['services']);

            $response->getBody()->write($model->toJSON());

            return $response->withStatus(201);

        } catch (\Exception $e) {
            if ($e->getCode() == 23000) {
                // Deal with duplicate key error
                throw new HttpException(409, $e->getMessage());
            }
            throw HttpException::internal($e);
        }
    }

    // Bad payload request
    throw HttpException::badRequest($validator);
};

//Define admin route callback
$app->post('/admin/place', $_func);
//Define route callback
$app->post('/place', $_func);
