<?php 

/**
 * Search place.
 */

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Valitron\Validator as Validator;
use App\Library\HttpException;

$_func = function (Request $request, Response $response, array $args) {

    $_get = $request->getQueryParams();

    $validator = new Validator($_get);
    $validator->rules(array(
        'required' => array(
            array('_page', true),
            array('_limit', true),
        ),
        'min' => array(
            array('_limit', 1), 
            array('_page', 0)
        ),
        'max' => array(
            array('_limit', 100)
        ) 
    ));

    // forbidden keys
    unset($_get['_id']);
    unset($_get['created_at']);
    unset($_get['updated_at']);
    unset($_get['description']);
    unset($_get['address_1']);
    unset($_get['address_2']);
    unset($_get['phone']);
    unset($_get['website_url']);
    unset($_get['timetable']);

    if ($validator->validate()) {

        $page = intval($_get['_page']);
        $limit = intval($_get['_limit']);
        $order = 'desc';
        $sort =  '_id';
        $skip = $page * $limit;

        if (isset($_get['_sort']) && in_array($_get['sort'], array('_id', 'created_at', 'updated_at', 'name', 'latitude', 'longitude', ))) {
            $sort = $_get['_sort'];
        }

        if (isset($_get['_order']) && in_array($_get['_order'], array('asc', 'desc'))) {
            $order = $_get['_order'];
        }

        if ($sort == '' || $order == '') {
            // string wrong formed
            throw new HttpException(400, 'Missing sort and/or order parameters');
        }

        $query = App\Models\Place::getCursor($_get, $request);
        $total = $query->count();
        /** @var App\Models\Place[] $models */
        $models = $query->orderBy($sort, $order)
                       ->skip($skip)
                       ->take($limit)
                       ->get();

        // Push data to payload
        $result = array();
        foreach($models as $model)
        {
            // Clone data
            $clone = json_decode(json_encode($model), true);
            // Populate relationships
            $clone['categories'] = $model->_categories;

            array_push($result, $clone);
        }

        $response->getBody()->write(json_encode(array(
            'page' => $page ,
            'limit' => $limit,
            'count' => count($result),
            'total' => $total,
            'items' => $result
        )));
        return $response->withStatus(200);
    }

    // string wrong formed
    throw HttpException::badRequest($validator);
};

//Define admin route callback
$app->get('/admin/place', $_func);
//Define route callback
$app->get('/place', $_func);
