<?php 

/**
 * Update place.
 */

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Valitron\Validator as Validator;
use App\Library\HttpException;

$_func = function (Request $request, Response $response, array $args) {

    //Authentication
    $this->get('session')->verifyCredentials($request);

    /** @var App\Models\Place $model */
    $model = App\Models\Place::find($args['id']);
    if ($model == null) {
        // Entity was not found
        throw new HttpException(404, 'Place not found');
    }

    //Ownership
    $this->get('session')->verifyOwnership($request, $model->owner);

    $data = $request->getParsedBody();

    //Form validation
    $validator = new Validator($data);
    $validator->rule('optional', 'name');
    $validator->rule('optional', 'description', true );
    $validator->rule('optional', 'categories')->rule('array', 'categories');
    $validator->rule('optional', 'address_1');
    $validator->rule('optional', 'address_2', true );
    $validator->rule('optional', 'phone', true );
    $validator->rule('optional', 'website_url', true );
    $validator->rule('optional', 'services', true )->rule('array', 'services');
    $validator->rule('optional', 'timetable', true );

    // For admins only
    if ($request->getAttribute('fromAdmin')) {
        $validator->rule('optional', 'disabled')->rule('boolean', 'disabled');
    } else {
        unset($data['disabled']);
    }

    if ($validator->validate()) {

        try {

            $model->fill($data);
            $model->save();

            $model->_categories()->detach();
            $model->_categories()->attach($data['categories']);

            $model->_services()->detach();
            $model->_services()->attach($data['services']);

            return $response->withStatus(204);

        } catch (\Exception $e) {

            if ($e->getCode() == 23000) {
                // Deal with duplicate key error
                throw new HttpException(409, $e->getMessage());
            }
            throw HttpException::internal($e);
        }
    }
    // Bad payload request
    throw HttpException::badRequest($validator);
};

//Define admin route callback
$app->patch('/admin/place/{id}', $_func);
//Define route callback
$app->patch('/place/{id}', $_func);
