<?php 

/**
 * Create user.
 * Returns the created user.
 */

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Valitron\Validator as Validator;
use App\Library\HttpException;

$_func = function (Request $request, Response $response, array $args) {

    $data = $request->getParsedBody();

    //Form validation
    $validator = new Validator($data);
    $validator->rule('required', 'email')->rule('email', 'email');
    $validator->rule('required', 'name');
    $validator->rule('required', 'password')->rule('lengthMin', 'password', 6);

    // For admins only
    if ($request->getAttribute('fromAdmin')) {
        $validator->rule('required', 'role');
        $validator->rule('required', 'banned')->rule('boolean', 'banned');
    }

    if ($validator->validate()) {

        try {

            // Init restricted fields
            if(!$request->getAttribute('fromAdmin')) {
                $data['role'] = 'user';
                $data['banned'] = false;
            }

            $model = new App\Models\User();
            $model->fill($data);

            // Init internal fields
            $model->setAttribute('created_at', date('Y-m-d H:i:s'));
            $model->setAttribute('updated_at', date('Y-m-d H:i:s'));
            $model->setAttribute('last_connected_at', date('Y-m-d H:i:s'));

            $model->save();

            $response->getBody()->write($model->toJSON());

            return $response->withStatus(201);

        } catch (\Exception $e) {
            if ($e->getCode() == 23000) {
                // Deal with duplicate key error
                throw new HttpException(409, $e->getMessage());
            }
            throw HttpException::internal($e);
        }
    }

    // Bad payload request
    throw HttpException::badRequest($validator);
};

//Define admin route callback
$app->post('/admin/user', $_func);
//Define route callback
$app->post('/user', $_func);
