<?php 

/**
 * Update user.
 */

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Valitron\Validator as Validator;
use App\Library\HttpException;

$_func = function (Request $request, Response $response, array $args) {

    //Authentication
    $this->get('session')->verifyCredentials($request);

    /** @var App\Models\User $model */
    $model = App\Models\User::find($args['id']);
    if ($model == null) {
        // Entity was not found
        throw new HttpException(404, 'User not found');
    }

    //Ownership
    $this->get('session')->verifyOwnership($request, $model->_id);

    $data = $request->getParsedBody();

    //Form validation
    $validator = new Validator($data);
    $validator->rule('optional', 'email')->rule('email', 'email');
    $validator->rule('optional', 'name');
    $validator->rule('optional', 'password')->rule('lengthMin', 'password', 6);

    // For admins only
    if ($request->getAttribute('fromAdmin')) {
        $validator->rule('optional', 'role');
        $validator->rule('optional', 'banned')->rule('boolean', 'banned');
    } else {
        unset($data['role']);
        unset($data['banned']);
    }

    if ($validator->validate()) {

        try {

            $model->fill($data);
            $model->save();

            return $response->withStatus(204);

        } catch (\Exception $e) {

            if ($e->getCode() == 23000) {
                // Deal with duplicate key error
                throw new HttpException(409, $e->getMessage());
            }
            throw HttpException::internal($e);
        }
    }
    // Bad payload request
    throw HttpException::badRequest($validator);
};

//Define admin route callback
$app->patch('/admin/user/{id}', $_func);
//Define route callback
$app->patch('/user/{id}', $_func);
